import sys

from mext.sendmail import *
from mext.htmlgen import *

from pasteob import *
from pasteob.debug.collect import Traceback
from pasteob.debug.utils import *

__all__ = ['EmailErrors', 'ShowTraceback']

@webob_middleware
def report_errors_mw(req, app, reporters):
    try:
        r = req.get_response(app)
        r.body
    except:
        exc_info = sys.exc_info()
        #try:
        reporter = None
        tb = Traceback.collect()
        rep = WebReport(req, tb)
        for reporter in reporters:
            r = reporter.report(rep)
##         except Exception, exc:
##             r = u"Secondary error has occurred while trying to report with %s: %s" % (
##                 unicode_safe(reporter),
##                 unicode_safe(exc)
##             )
##
        if r is None:
            r = u''
        if isinstance(r, (str, unicode)):
            r = ErrorResponse(500, r)
    return r





class Reporter(object):
    def wrap(self, app):
        return report_errors_mw(app, reporters=[self])


class ShowTraceback(Reporter):
    def report(self, rep):
        #return rep.tb_html
        html = HtmlDocument([TITLE[rep.tb_last_line], rep.tb_html])#, rep.req_summary_html])
        return Response.make(html, status=500)


class EmailErrors(Reporter):
    def __init__(self, send_to, send_from, subject_prefix="[error] "):
        self.send_to = send_to
        self.send_from = send_from
        self.subject_prefix = subject_prefix

    def report(self, rep):
        text = rep.tb_text
        text += '\n' + rep.req_summary_text

        html = HtmlDocument([rep.tb_html, rep.req_summary_html])

        email = AlternativesMessage([TextMessage(text), HtmlMessage(html)])
        email['from'] = self.send_from
        email['to'] = self.send_to
        subject = self.subject_prefix + rep.tb_last_line
        email['subject'] = truncate(subject, 100)
        Sendmail.send(email)
        return "Error report was successfully sent to administrator."












class Report(object):
    def __init__(self, tb):
        self.tb = tb
        self.tb_last_line = u'%s: %s' % (tb.exc_type.__name__, unicode_safe(tb.exc))

    @property
    def tb_text(self):
        lines = []#"Traceback:"]
        for frame in self.tb.frames:
            lines.append(u'File "%s", line %s in %s'
                % (frame.filename or '?', frame.line_no or '?', frame.func_name or '?')
            )
            source = frame.get_source_line()
            if source:
                lines.append('  ' + source.strip())
        lines.append(self.tb_last_line)
        return '\n'.join(lines)

    #@property
    #def tb_html(self):
    #    return [H3[self.tb_last_line], self.tb_html_min]

    @property
    def tb_html(self):
        r = PRE(cls='traceback')#['Traceback:', BR]
        for frame in self.tb.frames:
            r += SPAN(style='color:#C00')[
                'Module ', BOLD(title=frame.filename)[frame.module_name or '?'],
                ':', str(frame.line_no or '?'),
                ' in ', BOLD[frame.func_name or '?'],
                BR
            ]
            source = frame.get_source_line()
            if source:
                r += '  ', source.strip(), BR
        r += BOLD(style='font-size:140%')[self.tb_last_line]
        return r


class WebReport(Report):
    def __init__(self, req, tb):
        self.req = req
        super(WebReport, self).__init__(tb)

    @property
    def req_summary_text(self):
        r = '-' * 60
        r += '\nURL: %s\n' % self.req.url
        r += self.req_full_text
        return r

    @property
    def req_full_text(self):
        body_len = self.req.content_length
        big_request = (body_len or 0) > 1000
        r = self.req.__str__(skip_body=big_request)
        if big_request:
            r += '\n<body skipped (len=%s)>' % body_len
        return r


    @property
    def req_summary_html(self):
        r = DIV
        url = self.req.url
        r += HR
        r += CODE["URL: ", A(url)[url]]
        r += self.req_full_html
        return r

    @property
    def req_full_html(self):
        return PRE[self.req_full_text]



## def send_report(reporter, rep, errors, html=True):
##     try:
##         reporter.report(rep)
##     except:
##         msg = "Additionally an error occurred while sending the %s report:" % str(rep)
##         tb = traceback.format_exc()
##         if html:
##             errors.append(P[msg, PRE[tb]])
##         else:
##             errors.append(u"%s\n%s" % (msg, tb))
##
