from webob.statusreasons import status_reasons
from pasteob.base import wapp, Response
from mext.xmlgen import escape_text

__all__ = ['ErrorResponse', 'RedirectResponse']

_ctype_text = 'text/plain'
_ctype_html = 'text/html'

_string_codes = {
    'auth': 401,
    'forbidden': 403,
    'method': 405,
}

class ErrorResponse(object):
    def __init__(self, status, message=None, comment=None, **kw):
        if not isinstance(status, int):
            status = _string_codes[status]
        self.status = status
        self.title = u"%s %s" % (status, status_reasons[status])
        self.message = unicode(message or u'')
        self.comment = unicode(comment or u'')
        self.kw = kw


    @wapp
    def __call__(self, req):
        ctype = req.accept.best_match([_ctype_text, _ctype_html], _ctype_text)
        if ctype == _ctype_text:
            return self.text_response
        else:
            return self.html_response

    @property
    def text_response(self):
        body = self.message or self.title
        if self.comment:
            body += u'\n\n' + self.comment
        return Response(body, status=self.status, content_type=_ctype_text, **self.kw)

    @property
    def html_response(self):
        title = escape_text(self.title)
        body = u"<title>%s</title><h2>%s</h2>%s" % (
            title, title,
            escape_text(self.message)
        )
        if self.comment:
            body += u'\n<!--\n%s\n-->' % escape_text(self.comment)
        return Response(body, status=self.status, content_type=_ctype_html, **self.kw)

_redirect_types = {
    'temp': 307,
    'permanent': 301,
    'found': 302,
    'see other': 303,
}

class RedirectResponse(object):
    def __init__(self, status, location, message=None):
        if not isinstance(status, int):
            status = _redirect_types[status]
        self.status = status
        self.location = unicode(location)
        self.message = unicode(message or u"%s %s" % (status, status_reasons[status]))

    @wapp
    def __call__(self, req):
        body = self.message or self.title
        # unicode(req.uri.join(self.location))
        body += u'\n' + self.location
        return Response(body, status=self.status, content_type=_ctype_text, location=self.location)#, **self.kw)
